@@ -138,8 +138,7 @@
   (def-type-predicate-wrapper simple-rank-1-array-*-p)
   (def-type-predicate-wrapper simple-string-p)
   (def-type-predicate-wrapper stringp)
-  (def-type-predicate-wrapper vectorp)
-  (def-type-predicate-wrapper vector-nil-p))
+  (def-type-predicate-wrapper vectorp))
 
 #+(or x86 x86-64 arm arm64)
 (defun fixnum-mod-p (x limit)
@@ -234,25 +233,37 @@
            ((eq object nil) 'null)
            ((eq (sb-xc:symbol-package object) *keyword-package*) 'keyword)
            (t 'symbol)))
-    ((or array complex #+sb-simd-pack simd-pack #+sb-simd-pack-256 simd-pack-256)
-     (let ((sb-kernel::*unparse-allow-negation* nil))
-       (declare (special sb-kernel::*unparse-allow-negation*)) ; forward ref
-       (type-specifier (ctype-of object))))
+    (array
+     (let ((etype (specifier-type (array-element-type object))))
+       ;; Obviously :COMPLEXP is known to be T or NIL, but it's not allowed to
+       ;; return (NOT SIMPLE-ARRAY), so use :MAYBE in lieu of T.
+       (type-specifier
+        (make-array-type (array-dimensions object)
+                         :complexp (if (typep object 'simple-array) nil :maybe)
+                         :element-type etype
+                         :specialized-element-type etype))))
+    ((or complex #+sb-simd-pack simd-pack #+sb-simd-pack-256 simd-pack-256)
+     (type-specifier (ctype-of object)))
+    (simple-fun 'compiled-function)
     (t
-     (let* ((classoid (classoid-of object))
-            (name (classoid-name classoid)))
-       (if (%instancep object)
-           (case name
-             (sb-alien-internals:alien-value
-              `(alien
-                ,(sb-alien-internals:unparse-alien-type
-                  (sb-alien-internals:alien-value-type object))))
-             (t
-              (let ((pname (classoid-proper-name classoid)))
-                (if (classoid-p pname)
-                    (classoid-pcl-class pname)
-                    pname))))
-           name)))))
+     (let ((layout (layout-of object)))
+       (if (eq (get-lisp-obj-address layout) 0)
+            ;; An empty instance, e.g. created by with-output-to-string
+            'instance
+            (let* ((classoid (layout-classoid layout))
+                   (name (classoid-name classoid)))
+              (if (%instancep object)
+                  (case name
+                    (sb-alien-internals:alien-value
+                     `(alien
+                       ,(sb-alien-internals:unparse-alien-type
+                         (sb-alien-internals:alien-value-type object))))
+                    (t
+                     (let ((pname (classoid-proper-name classoid)))
+                       (if (classoid-p pname)
+                           (classoid-pcl-class pname)
+                           pname))))
+                  name)))))))
 
 ;;;; equality predicates
 
