@@ -74,9 +74,10 @@
                     target not-p ,type-codes
                     :value-tn-ref args)))))
 
+#-x86-64 ; defined in compiler/x86-64/type-vops for x86-64
 (define-type-vop fixnump
   #.fixnum-lowtags
-  #+(or x86-64 x86) simple-type-predicate) ;; save a register
+  #+(or x86) simple-type-predicate) ;; save a register
 
 (define-type-vop functionp (fun-pointer-lowtag))
 
@@ -113,7 +114,7 @@
 
 (define-type-vop simple-string-p
   (#+sb-unicode simple-character-string-widetag
-   simple-base-string-widetag simple-array-nil-widetag))
+   simple-base-string-widetag))
 
 (macrolet
     ((define-simple-array-type-vops ()
@@ -157,7 +158,7 @@
   (simple-array-widetag
    #+sb-unicode complex-character-string-widetag
    complex-base-string-widetag complex-bit-vector-widetag
-   complex-vector-widetag complex-array-widetag complex-vector-nil-widetag))
+   complex-vector-widetag complex-array-widetag))
 
 (define-type-vop simple-array-header-p
   (simple-array-widetag))
@@ -165,8 +166,7 @@
 (define-type-vop stringp
   (#+sb-unicode simple-character-string-widetag
    #+sb-unicode complex-character-string-widetag
-   simple-base-string-widetag complex-base-string-widetag
-   simple-array-nil-widetag complex-vector-nil-widetag))
+   simple-base-string-widetag complex-base-string-widetag))
 
 (define-type-vop base-string-p
   (simple-base-string-widetag complex-base-string-widetag))
@@ -174,9 +174,6 @@
 (define-type-vop bit-vector-p
   (simple-bit-vector-widetag complex-bit-vector-widetag))
 
-(define-type-vop vector-nil-p
-  (simple-array-nil-widetag complex-vector-nil-widetag))
-
 #+sb-unicode
 (define-type-vop character-string-p
   (simple-character-string-widetag complex-character-string-widetag))
