@@ -47,6 +47,7 @@
 
   ;; Return the address to which to jump when calling FDEFN,
   ;; which is either an fdefn or the name of an fdefn.
+  ;; FIXME: Shouldn't this go in x86-64-vm ?
   (defun sb-vm::fdefn-entry-address (fdefn)
     (let ((fdefn (if (fdefn-p fdefn) fdefn (find-or-create-fdefn fdefn))))
       (+ (get-lisp-obj-address fdefn)
@@ -73,6 +74,7 @@
          ;; CODE-OBJ must already be pinned in order to legally call this.
          ;; One call site that reaches here is below at MAKE-CORE-COMPONENT
          ;; and the other is LOAD-CODE, both of which pin the code.
+         ;; SYM is a little bit of a misnomer - it may be a generalized function name.
          (when (sb-vm:fixup-code-object
                  code-obj offset
                  (ecase flavor
@@ -86,13 +88,14 @@
                    #+sb-thread (:symbol-tls-index (ensure-symbol-tls-index sym))
                    (:layout (get-lisp-obj-address
                              (if (symbolp sym) (find-layout sym) sym)))
+                   (:layout-id (layout-id sym))
                    (:immobile-symbol (get-lisp-obj-address sym))
                    (:symbol-value (get-lisp-obj-address (symbol-global-value sym)))
                    #+immobile-code
                    (:named-call
-                    (when statically-link-p
-                      (push (cons offset sym) (elt preserved-lists 0)))
-                    (sb-vm::fdefn-entry-address sym))
+                    (prog1 (sb-vm::fdefn-entry-address sym) ; creates if didn't exist
+                      (when statically-link-p
+                        (push (cons offset (find-fdefn sym)) (elt preserved-lists 0)))))
                    #+immobile-code (:static-call (sb-vm::function-raw-address sym)))
                  kind flavor)
            (ecase kind
@@ -113,12 +116,10 @@
            (aver (not abs64-fixups)) ; no preserved 64-bit fixups
            (when (or abs-fixups rel-fixups)
              (setf (sb-vm::%code-fixups code-obj)
-                   (sb-c::pack-code-fixup-locs abs-fixups rel-fixups))))
+                   (sb-c:pack-code-fixup-locs abs-fixups rel-fixups))))
          (awhen (elt preserved-lists 4)
            (setf (gethash code-obj *allocation-point-fixups*)
                  (convert-alloc-point-fixups code-obj it)))
-         (awhen (aref preserved-lists 0)
-           (sb-vm::statically-link-code-obj code-obj it))
          ;; Assign all SIMPLE-FUN-SELF slots
          (dotimes (i (code-n-entries code-obj))
            (let ((fun (%code-entry-point code-obj i)))
@@ -130,8 +131,9 @@
                    (truly-the (unsigned-byte 32)
                      (get-lisp-obj-address #.(find-layout 'function))))))
          ;; And finally, make the memory range executable
-         #-(or x86 x86-64)
-         (sb-vm:sanctify-for-execution code-obj)))
+         #-(or x86 x86-64) (sb-vm:sanctify-for-execution code-obj)
+         ;; Return fixups amenable to static linking
+         (aref preserved-lists 0)))
 
   (defun apply-fasl-fixups (fop-stack code-obj n-fixups &aux (top (svref fop-stack 0)))
     (dx-let ((preserved (make-array 5 :initial-element nil)))
@@ -169,7 +171,8 @@
   (let* ((nbytes (code-object-size code))
          (boxed (code-header-words code)) ; word count
          (unboxed (- nbytes (ash boxed sb-vm:word-shift))) ; byte count
-         (copy (allocate-code-object :dynamic boxed unboxed)))
+         (copy (allocate-code-object
+                :dynamic (code-n-named-calls code) boxed unboxed)))
     (with-pinned-objects (code copy)
       (%byte-blt (code-instructions code) 0 (code-instructions copy) 0 unboxed)
       ;; copy boxed constants so that the fixup step (if needed) sees the 'fixups'
@@ -202,6 +205,9 @@
            (type index i))
   (let* ((info (leaf-info fun))
          (found (gethash info (core-object-entry-table object))))
+    ;; A  core component should not have cross-component references.
+    ;; If it could, then the entries would have placed into one component.
+    (aver found)
     (if found
         (setf (code-header-ref code-obj i) found)
         (push (cons code-obj i)
@@ -210,69 +216,141 @@
 
 ;;; Dump a component to core. We pass in the assembler fixups, code
 ;;; vector and node info.
+
+;;; Note that it is critical that the new code object not be movable after
+;;; copying in unboxed bytes and prior to fixing up those bytes.
+;;; Why: suppose the object has a jump table initially filled with addends of
+;;; 0x1000, 0x1100, 0x1200 representing the label offsets. If GC moves the object,
+;;; it adds the amount of movement to those labels. If it moves by, say +0x5000,
+;;; then the addends get mangled into 0x6000, 0x6100, 0x6200.
+;;; When we then add those addends to the virtual address of the code to
+;;; perform fixup application, the resulting addresses are all wrong.
+;
+;;; While GC might be made to use a heuristic to decide whether the label offsets
+;;; had been fixed up at all, it would be fragile nonetheless, because an offset
+;;; could theoretically resemble a valid address on machines where code resides
+;;; at low addresses in an object whose size is large. e.g. for an object which
+;;; spans the range 0x10000..0x30000 in memory, and the word 0x11000 in the jump
+;;; table, does that word represent an already-fixed-up label offset of 0x01000,
+;;; or an un-fixed-up value which needs to become 0x21000 ? It's ambiguous.
+;;; We could add 1 bit to the code object signifying that fixups had beeen applied
+;;; for the first time. But that complication is not needed, as long as we keep the
+;;; code pinned. That suffices because prior to copying in anything, all bytes
+;;; are 0, so the jump table count is 0.
+;;; Similar considerations pertain to x86[-64] fixups within the machine code.
+
+(defun assign-code-serialno (code-obj)
+  (let* ((serialno (ldb (byte (byte-size sb-vm::code-serialno-byte) 0)
+                        (atomic-incf sb-fasl::*code-serialno*)))
+         (insts (code-instructions code-obj))
+         (jumptable-word (sap-ref-word insts 0)))
+    (aver (zerop (ash jumptable-word -14)))
+    (setf (sap-ref-word insts 0) ; insert serialno
+          (logior (ash serialno (byte-position sb-vm::code-serialno-byte))
+                  jumptable-word))))
+
 (defun make-core-component (component segment length fixup-notes object)
   (declare (type component component)
            (type segment segment)
            (type index length)
            (list fixup-notes)
            (type core-object object))
-  (let ((debug-info (debug-info-for-component component)))
-    (let* ((2comp (component-info component))
-           (constants (ir2-component-constants 2comp))
-           (nboxed (align-up (length constants) sb-c::code-boxed-words-align))
-           (code-obj (allocate-code-object
-                      (component-mem-space component) nboxed length)))
-
+  (let* ((debug-info (debug-info-for-component component))
+         (2comp (component-info component))
+         (constants (ir2-component-constants 2comp))
+         (nboxed (align-up (length constants) sb-c::code-boxed-words-align))
+         (code-obj (allocate-code-object
+                    (component-mem-space component)
+                    (count-if (lambda (x) (typep x '(cons (eql :named-call))))
+                              constants)
+                    nboxed length))
+         (named-call-fixups
       ;; The following operations need the code pinned:
       ;; 1. copying into code-instructions (a SAP)
       ;; 2. apply-core-fixups and sanctify-for-execution
-      (with-pinned-objects (code-obj)
-        (let ((bytes (the (simple-array assembly-unit 1)
-                          (segment-contents-as-vector segment))))
-          ;; By design, until the last 2 unboxed bytes of CODE-OBJ contain a
-          ;; nonzero value, GC will not see any simple-funs therein.
-          (%byte-blt bytes 0 (code-instructions code-obj) 0 (length bytes)))
-        (apply-core-fixups fixup-notes code-obj))
+      ;; A very specific store order is necessary to allow using uninitialized memory
+      ;; pages for code. Storing of the debug-info slot does not need the code pinned,
+      ;; but that store must occur between steps 1 and 2.
+          (with-pinned-objects (code-obj)
+           (let ((bytes (the (simple-array assembly-unit 1)
+                             (segment-contents-as-vector segment))))
+             ;; Note that this does not have to take care to ensure atomicity
+             ;; of the store to the final word of unboxed data. Even if BYTE-BLT were
+             ;; interrupted in between the store of any individual byte, this code
+             ;; is GC-safe because we no longer need to know where simple-funs are embedded
+             ;; within the object to trace pointers. We *do* need to know where the funs
+             ;; are when transporting the object, but it's currently pinned.
+             (%byte-blt bytes 0 (code-instructions code-obj) 0 (length bytes))
+             ;; Serial# shares a word with the jump-table word count,
+             ;; so we can't assign serial# until after all raw bytes are copied in.
+             (assign-code-serialno code-obj))
+           ;; Enforce that the final unboxed data word is published to memory
+           ;; before the debug-info is set.
+           (sb-thread:barrier (:write))
+           ;; Until debug-info is assigned, it is illegal to create a simple-fun pointer
+           ;; into this object, because the C code assumes that the fun table is in an
+           ;; invalid/incomplete state (i.e. can't be read) until the code has debug-info.
+           ;; That is, C code can't deal with an interior code pointer until the fun-table
+           ;; is valid. This store must occur prior to calling %CODE-ENTRY-POINT, and
+           ;; applying fixups calls %CODE-ENTRY-POINT, so we have to do this before that.
+           (setf (%code-debug-info code-obj) debug-info)
+           (apply-core-fixups fixup-notes code-obj))))
 
       ;; Don't need code pinned now
-      (let* ((entries (ir2-component-entries 2comp))
-             (fun-index (length entries)))
-        (dolist (entry-info entries)
-          (let ((fun (%code-entry-point code-obj (decf fun-index)))
-                (w (+ sb-vm:code-constants-offset
-                      (* sb-vm:code-slots-per-simple-fun fun-index))))
-            (setf (code-header-ref code-obj (+ w sb-vm:simple-fun-name-slot))
-                  (entry-info-name entry-info)
-                  (code-header-ref code-obj (+ w sb-vm:simple-fun-arglist-slot))
-                  (entry-info-arguments entry-info)
-                  (code-header-ref code-obj (+ w sb-vm:simple-fun-source-slot))
-                  (entry-info-form/doc entry-info)
-                  (code-header-ref code-obj (+ w sb-vm:simple-fun-info-slot))
-                  (entry-info-type/xref entry-info))
-            (note-fun entry-info fun object))))
+      ;; (It will implicitly be pinned on the conservatively scavenged backends)
+    (let* ((entries (ir2-component-entries 2comp))
+           (fun-index (length entries)))
+      (dolist (entry-info entries)
+        (let ((fun (%code-entry-point code-obj (decf fun-index)))
+              (w (+ sb-vm:code-constants-offset
+                    (* sb-vm:code-slots-per-simple-fun fun-index))))
+          (setf (code-header-ref code-obj (+ w sb-vm:simple-fun-name-slot))
+                (entry-info-name entry-info)
+                (code-header-ref code-obj (+ w sb-vm:simple-fun-arglist-slot))
+                (entry-info-arguments entry-info)
+                (code-header-ref code-obj (+ w sb-vm:simple-fun-source-slot))
+                (entry-info-form/doc entry-info)
+                (code-header-ref code-obj (+ w sb-vm:simple-fun-info-slot))
+                (entry-info-type/xref entry-info))
+          (note-fun entry-info fun object))))
 
-      (push debug-info (core-object-debug-info object))
-      (setf (%code-debug-info code-obj) debug-info)
+    (push debug-info (core-object-debug-info object))
 
-      (do ((index sb-vm:code-constants-offset (1+ index)))
-          ((>= index (length constants)))
-        (let ((const (aref constants index)))
-            (etypecase const
-              (null)
-              (constant
-               (setf (code-header-ref code-obj index)
-                     (constant-value const)))
-              (list
-               (ecase (car const)
-                 (:entry
-                  (reference-core-fun code-obj index (cadr const) object))
-                 (:fdefinition
-                  (setf (code-header-ref code-obj index)
-                        (find-or-create-fdefn (cadr const))))
-                 (:known-fun
-                  (setf (code-header-ref code-obj index)
-                        (%coerce-name-to-fun (cadr const)))))))))))
-  (values))
+    (do ((index (+ sb-vm:code-constants-offset
+                   (* (length (ir2-component-entries 2comp))
+                      sb-vm:code-slots-per-simple-fun))
+                (1+ index)))
+        ((>= index (length constants)))
+      (let* ((const (aref constants index))
+             (kind (if (listp const) (car const) const)))
+        (case kind
+          (:entry
+           (reference-core-fun code-obj index (cadr const) object))
+          ((nil))
+          (t
+           (let ((referent
+                  (etypecase kind
+                   ((member :named-call :fdefinition)
+                    (find-or-create-fdefn (cadr const)))
+                   ((eql :known-fun)
+                    (%coerce-name-to-fun (cadr const)))
+                   (constant
+                    (constant-value const)))))
+             (if (eq kind :named-call)
+                 (set-code-fdefn code-obj index referent)
+                 (setf (code-header-ref code-obj index) referent)))))))
+    (when named-call-fixups
+      (sb-vm::statically-link-code-obj code-obj named-call-fixups))
+    code-obj))
+
+(defun set-code-fdefn (code index fdefn)
+  #+untagged-fdefns
+  (with-pinned-objects (fdefn)
+    (setf (code-header-ref code index)
+          (%make-lisp-obj (logandc2 (get-lisp-obj-address fdefn)
+                                    sb-vm:lowtag-mask))))
+  #-untagged-fdefns
+  (setf (code-header-ref code index) fdefn))
 
 ;;; Backpatch all the DEBUG-INFOs dumped so far with the specified
 ;;; SOURCE-INFO list. We also check that there are no outstanding
