@@ -10,26 +10,33 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
-(defun symbol-slot-ea (symbol slot &optional (size :qword))
-  (make-ea size :disp
-           (let ((offset (- (* slot n-word-bytes) other-pointer-lowtag)))
+(defun symbol-slot-ea (symbol slot)
+  (ea (let ((offset (- (* slot n-word-bytes) other-pointer-lowtag)))
              (if (static-symbol-p symbol)
                  (+ nil-value (static-symbol-offset symbol) offset)
-                 (make-fixup symbol :immobile-object offset)))))
+                 (make-fixup symbol :immobile-symbol offset)))))
 
-(defun gen-cell-set (ea value result)
+(defun gen-cell-set (ea value result &optional vop pseudo-atomic)
+  (when pseudo-atomic
+    ;; (SETF %FUNCALLABLE-INSTANCE-FUN) and (SETF %FUNCALLABLE-INSTANCE-INFO)
+    ;; pass in pseudo-atomic = T.
+    (pseudo-atomic ()
+     (inst push (ea-base ea))
+     (invoke-asm-routine 'call 'touch-gc-card vop)
+     (gen-cell-set ea value result))
+    (return-from gen-cell-set))
   (when (sc-is value immediate)
     (let ((bits (encode-value-if-immediate value)))
       (cond ((not result)
              ;; Try to move imm-to-mem if BITS fits
              (acond ((or (and (fixup-p bits)
                               ;; immobile-object fixups must fit in 32 bits
-                              (eq (fixup-flavor bits) :immobile-object)
+                              (eq (fixup-flavor bits) :immobile-symbol)
                               bits)
-                         (immediate32-p bits))
-                     (inst mov ea it))
+                         (plausible-signed-imm32-operand-p bits))
+                     (inst mov :qword ea it))
                     (t
                      (inst mov temp-reg-tn bits)
                      (inst mov ea temp-reg-tn)))
@@ -48,7 +55,7 @@
              (inst mov result bits)
              (inst mov ea result)
              (return-from gen-cell-set)))))
-  (inst mov ea value)
+  (inst mov :qword ea value) ; specify the size for when VALUE is an integer
   (when result
     ;; Ideally we would skip this move if RESULT is unused hereafter,
     ;; but unfortunately (NOT (TN-READS RESULT)) isn't equivalent
@@ -72,8 +79,7 @@
   (:variant-vars offset lowtag)
   (:policy :fast-safe)
   (:generator 4
-    (gen-cell-set (make-ea-for-object-slot object offset lowtag)
-                  value nil)))
+    (gen-cell-set (object-slot-ea object offset lowtag) value nil)))
 
 ;;; X86 special
 (define-vop (cell-xadd)
@@ -85,7 +91,7 @@
   (:policy :fast-safe)
   (:generator 4
     (move result value)
-    (inst xadd (make-ea-for-object-slot object offset lowtag) result :lock)))
+    (inst xadd :lock (object-slot-ea object offset lowtag) result)))
 
 (define-vop (cell-xsub cell-xadd)
   (:args (object)
@@ -101,7 +107,7 @@
      (t
       (move result value)
       (inst neg result)))
-    (inst xadd (make-ea-for-object-slot object offset lowtag) result :lock)))
+    (inst xadd :lock (object-slot-ea object offset lowtag) result)))
 
 (define-vop (atomic-inc-symbol-global-value cell-xadd)
   (:translate %atomic-inc-symbol-global-value)
@@ -153,22 +159,29 @@
                (inst jmp :nz err)
                (if const
                    (cond ((typep const '(signed-byte 32))
-                          (inst lea newval
-                                (make-ea :qword :base rax :disp const)))
+                          (inst lea newval (ea const rax)))
                          (t
                           (inst mov newval const)
                           (inst add newval rax)))
                    ,(if (eq inherit 'cell-xsub)
                         `(progn (move newval rax)
                                 (inst sub newval delta))
-                        `(inst lea newval
-                               (make-ea :qword :base rax :index delta))))
-               (inst cmpxchg
-                     (make-ea-for-object-slot cell ,slot list-pointer-lowtag)
-                     newval :lock)
+                        `(inst lea newval (ea rax delta))))
+               (inst cmpxchg :lock
+                     (object-slot-ea cell ,slot list-pointer-lowtag)
+                     newval)
                (inst jmp :ne retry)
                (inst mov result rax)))))))
   (def-atomic %atomic-inc-car cell-xadd cons-car-slot)
   (def-atomic %atomic-inc-cdr cell-xadd cons-cdr-slot)
   (def-atomic %atomic-dec-car cell-xsub cons-car-slot)
   (def-atomic %atomic-dec-cdr cell-xsub cons-cdr-slot))
+
+;; Atomically set a bit of an instance header word
+(define-vop (set-instance-hashed)
+  (:args (x :scs (descriptor-reg)))
+  (:generator 1
+    (inst or :lock :byte (ea (- 1 instance-pointer-lowtag) x)
+          ;; Bit index is 0-based. Subtract 8 since we're using the EA
+          ;; to select byte 1 of the header word.
+          (ash 1 (- stable-hash-required-flag 8)))))
