@@ -208,28 +208,16 @@ print_entry_points (struct code *code, FILE *f)
  * better not change. */
 
 struct call_frame {
-#ifndef LISP_FEATURE_ALPHA
         struct call_frame *old_cont;
-#else
-        u32 old_cont;
-#endif
         lispobj saved_lra;
         lispobj code;
         lispobj other_state[5];
 };
 
 struct call_info {
-#ifndef LISP_FEATURE_ALPHA
     struct call_frame *frame;
-#else
-    u32 frame;
-#endif
     int interrupted;
-#ifndef LISP_FEATURE_ALPHA
     struct code *code;
-#else
-    u32 code;
-#endif
     lispobj lra;
     int pc; /* Note: this is the trace file offset, not the actual pc. */
 };
@@ -312,11 +300,7 @@ call_info_from_context(struct call_info *info, os_context_t *context)
     }
     if (info->code != NULL)
         info->pc = pc - (uword_t) info->code -
-#ifndef LISP_FEATURE_ALPHA
             (HEADER_LENGTH(info->code->header) * sizeof(lispobj));
-#else
-            (HEADER_LENGTH(((struct code *)info->code)->header) * sizeof(lispobj));
-#endif
     else
         info->pc = 0;
 }
@@ -486,17 +470,13 @@ void
 describe_thread_state(void)
 {
     struct thread *thread = arch_os_get_current_thread();
-    struct interrupt_data *data = thread->interrupt_data;
+    struct interrupt_data *data = &thread_interrupt_data(thread);
 #ifndef LISP_FEATURE_WIN32
     sigset_t mask;
-    get_current_sigmask(&mask);
-    printf("Signal mask:\n");
-    printf(" SIGALRM = %d\n", sigismember(&mask, SIGALRM));
-    printf(" SIGINT = %d\n", sigismember(&mask, SIGINT));
-    printf(" SIGPROF = %d\n", sigismember(&mask, SIGPROF));
-#ifdef SIG_STOP_FOR_GC
-    printf(" SIG_STOP_FOR_GC = %d\n", sigismember(&mask, SIG_STOP_FOR_GC));
-#endif
+    char string[180];
+    thread_sigmask(SIG_BLOCK, 0, &mask);
+    sigset_tostring(&mask, string, sizeof string);
+    if (string[0]) printf("Signal mask: %s\n", string);
 #endif
     printf("Specials:\n");
     printf(" *GC-INHIBIT* = %s\n", (read_TLS(GC_INHIBIT, thread) == T) ? "T" : "NIL");
@@ -619,15 +599,16 @@ static boolean __attribute__((unused)) print_lisp_fun_name(char* pc)
   return 0;
 }
 
-#ifdef LISP_FEATURE_LIBUNWIND_BACKTRACE
+#ifdef LISP_FEATURE_BACKTRACE_ON_SIGNAL
 #define UNW_LOCAL_ONLY
+#ifdef HAVE_LIBUNWIND
 #include <libunwind.h>
+#endif
 #include "genesis/thread-instance.h"
 #include "genesis/mutex.h"
 static __attribute__((unused))int backtrace_completion_pipe[2] = {-1,-1};
 void libunwind_backtrace(struct thread *th, os_context_t *context)
 {
-    char procname[100];
     fprintf(stderr, "Lisp thread @ %p, tid %d", th, (int)th->os_kernel_tid);
 #ifdef LISP_FEATURE_SB_THREAD
     // the TLS area is not used if #-sb-thread. And if so, it must be "main thred"
@@ -656,6 +637,8 @@ void libunwind_backtrace(struct thread *th, os_context_t *context)
         putc('\n', stderr);
     }
 #endif
+#ifdef HAVE_LIBUNWIND
+    char procname[100];
     unw_cursor_t cursor;
     // "unw_init_local() is thread-safe as well as safe to use from a signal handler."
     // "unw_get_proc_name() is thread-safe. If cursor cp is in the local address-space,
@@ -679,6 +662,11 @@ void libunwind_backtrace(struct thread *th, os_context_t *context)
             fprintf(stderr, " %p ?\n", pc);
         }
     } while (unw_step(&cursor));
+#else
+    // If you don't have libunwind, this will almost surely not work,
+    // because we can't figure out how to get backwards past a signal frame.
+    log_backtrace_from_fp((void*)*os_context_fp_addr(context), 100, 0, stderr);
+#endif
 }
 void backtrace_lisp_threads(int __attribute__((unused)) signal,
                                    siginfo_t __attribute__((unused)) *info,
@@ -693,8 +681,9 @@ void backtrace_lisp_threads(int __attribute__((unused)) signal,
     struct thread *th;
     int nthreads = 0;
     for_each_thread(th) { ++nthreads; }
-    fprintf(stderr, "Caught backtrace-all signal in tid %d, %d threads\n",
-            (int)arch_os_get_current_thread()->os_kernel_tid, nthreads);
+    if (signal)
+        fprintf(stderr, "Caught backtrace-all signal in tid %d, %d threads\n",
+                (int)arch_os_get_current_thread()->os_kernel_tid, nthreads);
     // Would be nice if we could forcibly stop all the other threads,
     // but pthread_mutex_trylock is not safe to use in a signal handler.
     if (nthreads > 1) {
@@ -718,4 +707,36 @@ void backtrace_lisp_threads(int __attribute__((unused)) signal,
     libunwind_backtrace(arch_os_get_current_thread(), context);
 #endif
 }
+static int watchdog_pipe[2] = {-1,-1};
+static pthread_t watchdog_tid;
+static void* watchdog_thread(void* arg) {
+    struct timeval timeout;
+    fd_set fds;
+    FD_ZERO(&fds);
+    FD_SET(watchdog_pipe[0], &fds);
+    timeout.tv_sec = (long)arg;
+    timeout.tv_usec = 0;
+    int nfds = select(watchdog_pipe[0]+1, &fds, 0, 0, &timeout);
+    if (nfds == 0) {
+        // Ensure this message comes out in one piece even if nothing following it does.
+        char msg[] = "Watchdog timer expired\n"; write(2, msg, sizeof msg-1);
+        backtrace_lisp_threads(0, 0, 0);
+        _exit(1); // cause the test suite to exit with failure
+    }
+    return 0;
+}
+void start_watchdog(int sec) {
+    if (pipe(watchdog_pipe)) lose("Can't make watchdog pipe");
+    pthread_create(&watchdog_tid, 0, watchdog_thread, (void*)(long)sec);
+    char msg[] = "Started watchdog thread\n"; write(2, msg, sizeof msg-1);
+}
+void stop_watchdog() {
+    char c[1] = {0};
+    write(watchdog_pipe[1], c, 1);
+    close(watchdog_pipe[1]);
+    void* result;
+    pthread_join(watchdog_tid, &result);
+    close(watchdog_pipe[0]);
+    watchdog_pipe[0] = watchdog_pipe[1] = -1;
+}
 #endif
